﻿using HeuristicLab.Algorithms.DataAnalysis;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Problems.GaussianProcessTuning {
  [StorableClass]
  [Item("Gaussian Process Configuration Evaluator", "Evaluator for the Gaussian process Configuration problem.")]
  public class Evaluator : SingleSuccessorOperator, ISingleObjectiveEvaluator {

    private const string QualityParameterName = "Quality";
    private const string GaussianProcessConfigurationParameterName = "GaussianProcessConfiguration";
    private const string InterpreterParameterName = "Interpreter";
    private const string ProblemDataParameterName = "ProblemData";
    private const string SolutionParameterName = "Solution";

    public ILookupParameter<DoubleValue> QualityParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters[QualityParameterName]; }
    }
    public ILookupParameter<IGaussianProcessSolution> SolutionParameter {
      get { return (ILookupParameter<IGaussianProcessSolution>)Parameters[SolutionParameterName]; }
    }
    public ILookupParameter<ISymbolicExpressionTree> GaussianProcessConfigurationParameter {
      get { return (ILookupParameter<ISymbolicExpressionTree>)Parameters[GaussianProcessConfigurationParameterName]; }
    }
    public ILookupParameter<Interpreter> InterpreterParameter {
      get { return (ILookupParameter<Interpreter>)Parameters[InterpreterParameterName]; }
    }
    public ILookupParameter<IRegressionProblemData> ProblemDataParameter {
      get { return (ILookupParameter<IRegressionProblemData>)Parameters[ProblemDataParameterName]; }
    }

    [StorableConstructor]
    protected Evaluator(bool deserializing) : base(deserializing) { }
    protected Evaluator(Evaluator original, Cloner cloner)
      : base(original, cloner) {
    }

    public Evaluator() {
      Parameters.Add(new LookupParameter<DoubleValue>(QualityParameterName, "The solution quality of the Gaussian process configuration."));
      Parameters.Add(new LookupParameter<ISymbolicExpressionTree>(GaussianProcessConfigurationParameterName, "The Gaussian process configuration to evaluate represented as symbolic expression tree."));
      Parameters.Add(new LookupParameter<Interpreter>(InterpreterParameterName, "The interpreter to use for evaluating Gaussian process configurations represented as trees."));
      Parameters.Add(new LookupParameter<IRegressionProblemData>(ProblemDataParameterName, "To regression problem data for which to learn a Gaussian process."));
      Parameters.Add(new LookupParameter<IGaussianProcessSolution>(SolutionParameterName, "The solution produced by Gaussian process regression algorithm"));
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new Evaluator(this, cloner);
    }
    public override IOperation Apply() {
      ISymbolicExpressionTree tree = GaussianProcessConfigurationParameter.ActualValue;

      double quality;
      IGaussianProcessSolution solution;
      InterpreterParameter.ActualValue.EvaluateGaussianProcessConfiguration(tree, ProblemDataParameter.ActualValue, out quality, out solution);

      QualityParameter.ActualValue = new DoubleValue(quality);
      if (solution != null)
        SolutionParameter.ActualValue = solution;
      return base.Apply();
    }
  }
}
